"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
exports.default = {
    tagName: 'table',
    getCategories(element) {
        return ['flow content', 'palpable content'];
    },
    validateContext(element) {
        const result = [];
        return result;
    },
    validateContent(element) {
        const result = [];
        const { children } = element;
        result.push(...(0, util_1.validateChildrenSequence)({
            desc: [
                'optionally a caption element',
                'followed by zero or more colgroup elements',
                'followed optionally by a thead element',
                'followed optionally by a tfoot element',
                'followed by either zero or more tbody elements or one or more tr elements',
                'followed optionally by a tfoot element',
                'optionally intermixed with one or more script-supporting elements'
            ].join(', '),
            filter: (0, util_1.isNotCategory)('script-supporting element'),
            sequence: [
                ['caption', '?'],
                ['colgroup', '*'],
                ['thead', '?'],
                ['tfoot', '?'],
                ['tbody|tr', '*'],
                ['tfoot', '?']
            ]
        }, element));
        if (children.some((0, util_1.isTag)('tbody'))
            && children.some((0, util_1.isTag)('tr'))) {
            result.push({
                expect: 'containing either tbody elements or tr elements',
                got: 'tbody and tr',
                target: element
            });
        }
        if (children.filter((0, util_1.isTag)('tfoot')).length > 1) {
            result.push({
                expect: 'containing only one tfoot element child in total',
                target: element
            });
        }
        return result;
    }
};
//# sourceMappingURL=table.js.map