"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
exports.default = {
    tagName: 'tfoot',
    getCategories(element) {
        return [];
    },
    validateContext(element) {
        const result = [];
        if (element.parentElement) {
            if ((0, util_1.isNotTag)('table', element.parentElement)) {
                result.push({
                    expect: 'as a child of an table element',
                    got: (0, util_1.getNodeInfo)(element.parentElement),
                    target: element
                });
            }
            for (let next = element; (next = next.nextElementSibling);) {
                if ((0, util_1.isTag)('caption|colgroup|thead', next)) {
                    result.push({
                        expect: 'after caption, colgroup, and thead elements',
                        target: element
                    });
                    break;
                }
            }
            let prev = element;
            let hasPrevTbodyTrElements = false;
            for (prev = element; (prev = prev.previousElementSibling);) {
                if ((0, util_1.isTag)('tbody|tr', prev)) {
                    hasPrevTbodyTrElements = true;
                    break;
                }
            }
            let hasNextTbodyTrElements = false;
            for (prev = element; (prev = prev.nextElementSibling);) {
                if ((0, util_1.isTag)('tbody|tr', prev)) {
                    hasNextTbodyTrElements = true;
                    break;
                }
            }
            if (hasPrevTbodyTrElements && hasNextTbodyTrElements) {
                result.push({
                    expect: 'either before any tbody and tr elements, or after any tbody and tr elements',
                    target: element
                });
            }
            if (element.parentElement.children.filter((0, util_1.isTag)('tfoot')).length > 1) {
                result.push({
                    expect: 'there are no other tfoot elements that are children of the table element',
                    target: element
                });
            }
        }
        return result;
    },
    validateContent(element) {
        const result = [];
        result.push(...(0, util_1.validateChildrenSequence)({
            desc: 'zero or more tr and script-supporting elements',
            sequence: [['tr|category:script-supporting element', '*']]
        }, element));
        return result;
    }
};
//# sourceMappingURL=tfoot.js.map