"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
exports.default = {
    tagName: 'tr',
    getCategories(element) {
        return [];
    },
    validateContext(element) {
        const result = [];
        if (element.parentElement
            && (0, util_1.isNotTag)('thead|tbody|tfoot|table', element.parentElement)) {
            result.push({
                expect: 'as a child of a thead element, a tbody element, a tfoot element or a table element',
                got: (0, util_1.getNodeInfo)(element.parentElement),
                target: element
            });
        }
        if ((0, util_1.isTag)('table', element.parentElement)) {
            for (let next = element; (next = next.nextElementSibling);) {
                if ((0, util_1.isTag)('caption|colgroup|thead', next)) {
                    result.push({
                        expect: 'after any caption, colgroup, and thead elements',
                        target: element
                    });
                    break;
                }
            }
            if (element.parentElement.children.filter((0, util_1.isTag)('tbody')).length > 0) {
                result.push({
                    expect: 'there are no tbody elements that are children of the table element',
                    target: element
                });
            }
        }
        return result;
    },
    validateContent(element) {
        const result = [];
        result.push(...(0, util_1.validateChildrenSequence)({
            desc: 'zero or more td, th, and script-supporting elements',
            sequence: [['td|th|category:script-supporting element', '*']]
        }, element));
        return result;
    }
};
//# sourceMappingURL=tr.js.map