"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNot = exports.is = exports.isType = exports.getContentModelParent = exports.isTransparentContentModel = exports.validateChildrenSequence = exports.validateCategory = exports.walkDescendants = exports.getAncestors = exports.isNotTag = exports.isTag = exports.isNotCategory = exports.isCategory = exports.getSequenceInfo = exports.getNodeCategoriesInfo = exports.getNodeInfo = exports.getCategories = exports.getRule = exports.walk = exports.not = exports.NodeType = exports.Node = exports.extend = void 0;
const lodash_1 = require("lodash");
const dom_1 = require("@mewjs/dom");
const rules_1 = require("./rules");
var lodash_2 = require("lodash");
Object.defineProperty(exports, "extend", { enumerable: true, get: function () { return lodash_2.extend; } });
var dom_2 = require("@mewjs/dom");
Object.defineProperty(exports, "Node", { enumerable: true, get: function () { return dom_2.Node; } });
var NodeType;
(function (NodeType) {
    NodeType[NodeType["ELEMENT_NODE"] = 1] = "ELEMENT_NODE";
    NodeType[NodeType["ATTRIBUTE_NODE"] = 2] = "ATTRIBUTE_NODE";
    NodeType[NodeType["TEXT_NODE"] = 3] = "TEXT_NODE";
    NodeType[NodeType["CDATA_SECTION_NODE"] = 4] = "CDATA_SECTION_NODE";
    NodeType[NodeType["ENTITY_REFERENCE_NODE"] = 5] = "ENTITY_REFERENCE_NODE";
    NodeType[NodeType["ENTITY_NODE"] = 6] = "ENTITY_NODE";
    NodeType[NodeType["PROCESSING_INSTRUCTION_NODE"] = 7] = "PROCESSING_INSTRUCTION_NODE";
    NodeType[NodeType["COMMENT_NODE"] = 8] = "COMMENT_NODE";
    NodeType[NodeType["DOCUMENT_NODE"] = 9] = "DOCUMENT_NODE";
    NodeType[NodeType["DOCUMENT_TYPE_NODE"] = 10] = "DOCUMENT_TYPE_NODE";
    NodeType[NodeType["DOCUMENT_FRAGMENT_NODE"] = 11] = "DOCUMENT_FRAGMENT_NODE";
})(NodeType = exports.NodeType || (exports.NodeType = {}));
function not(method) {
    return function is(...args) {
        const returnValue = method(...args);
        return typeof returnValue === 'function'
            ? not(returnValue)
            : !returnValue;
    };
}
exports.not = not;
function walk(root, handler, childrenKey = 'children') {
    handler(root);
    (root[childrenKey] || []).forEach(child => {
        (0, dom_1.isTag)(child) && walk(child, handler, childrenKey);
    });
}
exports.walk = walk;
function getRule(element) {
    return rules_1.rules[element.tagName.toLowerCase()];
}
exports.getRule = getRule;
function getCategories(element) {
    const rule = getRule(element);
    return rule
        ? rule.getCategories(element)
        : [];
}
exports.getCategories = getCategories;
function getNodeInfo(element) {
    return `<${element.tagName.toLowerCase()}>`;
}
exports.getNodeInfo = getNodeInfo;
function getNodeCategoriesInfo(el) {
    const element = el;
    const categories = getCategories(element);
    return `${getNodeInfo(element)} ( ${categories.length
        ? categories.join(' | ')
        : 'none'} )`;
}
exports.getNodeCategoriesInfo = getNodeCategoriesInfo;
function getSequenceInfo(sequence) {
    return `[ ${sequence.map(child => child.tagName.toLowerCase()).join(', ')} ]`;
}
exports.getSequenceInfo = getSequenceInfo;
exports.isCategory = (0, lodash_1.curry)((expect, el) => {
    const element = el;
    if (!getRule(element)) {
        return true;
    }
    const expects = expect.split('|');
    const categories = getCategories(element);
    return expects.some(expect => categories.includes(expect));
});
exports.isNotCategory = not(exports.isCategory);
exports.isTag = (0, lodash_1.curry)((tag, element) => {
    const el = element;
    return (el === null || el === void 0 ? void 0 : el.tagName) ? tag.split('|').includes(el.tagName.toLowerCase()) : false;
});
exports.isNotTag = not(exports.isTag);
function getAncestors(element) {
    const ancestors = [];
    let parent;
    while (element.parentElement) {
        parent = element.parentElement;
        ancestors.push(parent);
        element = parent;
    }
    return ancestors;
}
exports.getAncestors = getAncestors;
function walkDescendants(element, handler) {
    walk(element, (target) => {
        if (target !== element) {
            handler(target);
        }
    });
}
exports.walkDescendants = walkDescendants;
function validateCategory(expect, elements) {
    return elements.reduce((result, target) => {
        if ((0, exports.isNotCategory)(expect, target)) {
            result.push({
                expect,
                got: getNodeCategoriesInfo(target),
                target
            });
        }
        return result;
    }, []);
}
exports.validateCategory = validateCategory;
function validateChildrenSequence(expect, element) {
    let children = element.children.slice();
    if (expect.filter) {
        children = children.filter(node => expect.filter(node));
    }
    function createElementValidator(expectString) {
        const expects = expectString.split('|');
        return function validator(element) {
            return element && expects.reduce((valid, expect) => {
                let check = exports.isTag;
                const prefix = 'category:';
                if (expect.startsWith(prefix)) {
                    expect = expect.slice(prefix.length);
                    check = exports.isCategory;
                }
                return valid || check(expect, element);
            }, false);
        };
    }
    function unExpect(unexpected) {
        return [
            {
                expect: expect.desc,
                got: getSequenceInfo(element.children),
                target: element
            }
        ];
    }
    for (let i = 0, l = expect.sequence.length, expectInfo, validate, quantifier; i < l; i++) {
        expectInfo = expect.sequence[i];
        validate = typeof expectInfo[0] === 'function'
            ? expectInfo[0]
            : createElementValidator(expectInfo[0]);
        quantifier = expectInfo[1];
        switch (quantifier) {
            case '?':
                if (validate(children[0])) {
                    children.shift();
                }
                break;
            case '*':
                while (validate(children[0])) {
                    children.shift();
                }
                break;
            case '+':
                if (!validate(children[0])) {
                    return unExpect(children[0]);
                }
                do {
                    children.shift();
                } while (validate(children[0]));
                break;
            default:
                let n = parseInt(quantifier);
                while (n--) {
                    if (!validate(children[0])) {
                        return unExpect(children[0]);
                    }
                    children.shift();
                }
                break;
        }
    }
    return children.length ? unExpect(children[0]) : [];
}
exports.validateChildrenSequence = validateChildrenSequence;
const TRANSPARENT_TAGS = new Set([
    'audio',
    'video',
    'a',
    'del',
    'ins',
    'map',
    'object'
]);
function isTransparentContentModel({ tagName }) {
    return TRANSPARENT_TAGS.has(tagName.toLowerCase());
}
exports.isTransparentContentModel = isTransparentContentModel;
function getContentModelParent(element) {
    while (element.parentElement) {
        if (!isTransparentContentModel(element.parentElement)) {
            return element.parentElement;
        }
        element = element.parentElement;
    }
    return null;
}
exports.getContentModelParent = getContentModelParent;
const defines = {
    'inter-element whitespace': function (given) {
        return !given
            || (given.nodeType === NodeType.TEXT_NODE
                && /^[\u0020\u0009\u000a\u000c\u000d]*$/.test(given.textContent));
    },
    'media element': function (given) {
        return !!given
            && ['audio', 'video'].includes(given.tagName.toLowerCase());
    }
};
function isType(name, given) {
    const define = defines[name];
    return define
        ? define(given)
        : false;
}
exports.isType = isType;
exports.is = (0, lodash_1.curry)(isType, 2);
exports.isNot = not(exports.is);
//# sourceMappingURL=util.js.map